-- copyright benjamin bober, 2020
qualityoflife = qualityoflife or {};
local q = qualityoflife;

local partyMembers, raidMembers;
local oldPartyMembers, oldRaidMembers;
local lastJoinedMember, lastLeftMember;

function q:GetCharName()
	return GetUnitName("player");
end

function q:GetPartyMembers()
	if not partyMembers then
		partyMembers = {};
		local plist = GetHomePartyInfo();

		if plist then
			local n = 1;
			for _, v in pairs(plist) do
				partyMembers[n] = v;
				n = n + 1;
			end
		end
	end
	return partyMembers;
end

function q:GetRaidMembers()
	if not raidMembers then
		raidMembers = { [0] = {} };
		local noRaidMembers = GetNumGroupMembers();

		if noRaidMembers then
			for n = 1, noRaidMembers do
				local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(n);

				-- table of all raid members by id
				raidMembers[0][n] = name;

				-- table of all raid members, mapped by subgroups
				if not raidMembers[subgroup] then
					raidMembers[subgroup] = {};
				end
				local subGroupId = table.getn(raidMembers[subgroup]) + 1;
				raidMembers[subgroup][subGroupId] = name;

				n = n + 1;
			end
		end
	end
	return raidMembers;
end

function q:ResetMemberInfo()
	oldPartyMembers, oldRaidMembers = q:Deepcopy(q:GetPartyMembers()), q:Deepcopy(q:GetRaidMembers());
	partyMembers, raidMembers = nil, nil;
	lastJoinedMember, lastLeftMember = nil, nil;
end

function q:GetLastJoinedMember()
	if not lastJoinedMember then
		if table.getn(oldPartyMembers) < table.getn(q:GetPartyMembers()) then
			-- party member joined
			for id, name in ipairs(q:GetPartyMembers()) do
				if not q:TableContains(oldPartyMembers, name) then
					lastJoinedMember = name;
					return;
				end
			end
		elseif table.getn(oldRaidMembers[0]) < table.getn(q:GetRaidMembers()[0]) then
			-- raid member joined
			for id, name in ipairs(q:GetRaidMembers()[0]) do
				if not q:TableContains(oldRaidMembers[0], name) then
					lastJoinedMember = name;
					return;
				end
			end
		end
	end
	return lastJoinedMember;
end

function q:GetLastLeftMember()
	if not lastLeftMember then
		if table.getn(oldPartyMembers) > table.getn(q:GetPartyMembers()) then
			-- party member left
			for id, name in ipairs(oldPartyMembers) do
				if not q:TableContains(q:GetPartyMembers(), name) then
					lastLeftMember = name;
					return;
				end
			end
		elseif table.getn(oldRaidMembers[0]) > table.getn(q:GetRaidMembers()[0]) then
			-- raid member left
			for id, name in ipairs(oldRaidMembers[0]) do
				if not q:TableContains(q:GetRaidMembers()[0], name) then
					lastLeftMember = name;
					return;
				end
			end
		end
	end
	return lastLeftMember;
end

function q:GetBattleNetTag()
	if BNConnected() then
		local presenceID, toonID, currentBroadcast, bnetAFK, bnetDND = BNGetInfo();
		return toonID;
	end
end

function q:SendAddonMessage(addon, message, channel, target)
	addon = addon or q;
	C_ChatInfo.SendAddonMessage(addon.name, message, channel, target);
end